export interface XApiObjectSchema {
  definition: Object
  consented?: boolean
  defaultConsent: boolean
  label: string
  id: string
}

export interface XApiObjectConsented {
  definition: Object
  consented: boolean
  defaultConsent: boolean
  label: string
  id: string
}

interface XApiVerb {
}

export interface XApiVerbSchema {
  id: string
  label: string
  description: string
  defaultConsent: boolean
  objects: XApiObjectSchema[]
}

export interface XApiVerbConsented extends XApiVerbSchema {
  consented: boolean
  objects: XApiObjectConsented[]
}

export interface ConsentGroupSchema {
  id: string
  label: string
  description: string
  purposeOfCollection: string
  showVerbDetails: boolean
  verbs: XApiVerbSchema[]
  isDefault: boolean
}

export const instanceOfXApiVerbConsented = (object: any): object is XApiObjectConsented => {
  return 'consented' in object
}

export interface ConsentGroupConsented {
  id: string
  label: string
  description: string
  purposeOfCollection: string
  showVerbDetails: boolean
  verbs: XApiVerbConsented[]
  isDefault: boolean
}

interface ConsentDeclaration {
  id: string
  description: string
}

export interface ProviderConsent {
  id: string
  name: string
  superseded_by: number | null
  createdAt: string
  definition: ConsentDeclaration[]
  supersedes?: ProviderConsent
}

export interface ProviderSchemaDefinition {
  id: string
  name: string
  description: string
  groups: ConsentGroupSchema[]
  essential_verbs: XApiVerbSchema[] // verbs which can be collected without the user consent
}

export interface ProviderSchema extends ConsentDeclaration {
  superseded_by: number | null
  createdAt: string
  groups: ConsentGroupSchema[]
  essential_verbs: XApiVerbSchema[] // verbs which can be collected without the user consent
  definition: ProviderSchemaDefinition
}

export interface UserConsent extends ConsentDeclaration {
  id: string
  groups: ConsentGroupConsented[]
  essential_verbs: XApiVerbSchema[] // verbs which can be collected without the user consent
  created?: Date
}

export interface UserConsentVerbs {
  providerId: ProviderId
  providerSchemaId: string
  verbs: { id: string; consented?: boolean; objects: string }[]
}

export type ProviderId = number

/**
 * Iterates over each verb and object of a provider schema and sets the consented field to the default consent
 * @param providerSchema
 * @returns
 */
export const providerSchemaToUserConsent = (providerSchema: ProviderSchema): UserConsent => {
  return {
    id: providerSchema.id,
    description: providerSchema.description,
    groups: providerSchema.groups.map((group) => ({
      ...group,
      verbs: group.verbs.map((verb) => ({
        ...verb,
        consented: verb.defaultConsent,
        objects: verb.objects ? verb.objects.map((object) => ({
          ...object,
          consented: object.defaultConsent
        })) : []
      }))
    })),
    essential_verbs: providerSchema.essential_verbs
  }
}
